// Module Name: maze.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
// 
// This is the main program for the walking maze assignment.
//  
// Usage:
// 
// Run the program executable and it will automatically start with the
// checkerboard pattern loaded. The following commands are valid:
// 
// < and > : Rotates the view +- 10 degrees at a time
// U and D : Allows rotation of the view up or down by +- 10 degrees max 30.
// { and } : Moves left and right 0.2 squares
// + and - : Moves forward or backward 0.2 squares
// 
////////////////////////////////////////////////////////////////////////////

// Eeeewwwww
//#include <windows.h>

// System includes
#include <GL/gl.h>
#include <GL/glut.h>
#include <iostream.h>
#include <fstream.h>

// Object Includes
#ifndef MAZE
#include "maze.h"
#endif

#ifndef WALL
#include "wall.h"
#endif

#ifndef CAMERA
#include "../classes/camera.h"
#endif

// Height and width of the active window
int windowHeight, windowWidth;

// Mouse status variables
int mouseClickX, mouseClickY;
int leftMouseDown = 0;

// Object pointers
Maze* maze;
Wall* wall;
Camera* camera;

// Configure projection information
void init(void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   camera->on();

   // Make sure we clip hidden surfaces!
   glEnable(GL_DEPTH_TEST);
}

// Redraw the whole scene
void display(void)
{
	// Set the active projection
	camera->lookAtScene(); 

	// Rotate so that we're looking into the maze
	camera->rotateAxis(90, 1, 0, 0);

	// Draw whatever maze we're working with
	maze->draw();

	// Swap and flush the buffers
	glutSwapBuffers();

}

// Reshape the window - save the parameters
void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);

   windowWidth = w;
   windowHeight = h; 
}

// Loads the maze from the files created by mazemaker into a 2D array
void loadMaze()
{
	maze->generateMaze();
}

// Keyboard handler
void keyboard(unsigned char key, int x, int y)
{
	double xR, yR, zR;
	double xS, yS, zS;
	maze->getRotation(xR, yR, zR);
	maze->getScale(xS, yS, zS);


   switch (key) {
      case 27:
         exit(0);
         break;

      case 'l':
         loadMaze();
         display(); // redraw
         break;   

      case '+':
 		 camera->moveForward(0.2); 
         display();
         break;

      case '-':
		 camera->moveBackward(0.2);
         display();
         break;

      case '{':
 		 camera->moveRight(0.2);
         display();
         break;

      case '}':
		 camera->moveLeft(0.2);
         display();
         break;

      case '>':
 		 camera->rotateEye(10.,0.0);
         display();
         break;

      case '<':
		 camera->rotateEye(-10.0,0.0);
         display();
         break;                  

      case 'U':
 		 camera->rotateEye(0.,10.0);
         display();
         break;

      case 'D':
		 camera->rotateEye(0.0,-10.0);
         display();
         break; 

   }
}

// Mouse motion callback
void mouseMove(int x, int y)
{
	// Not used
}

// Snags mouse clicks for the mouse move callback
void mouse(int button, int state, int mouseX, int mouseY)
{
	// Snag the origin point for the control drag
    if (button == GLUT_LEFT_BUTTON)
	{
   		if (state == GLUT_DOWN)
   		{
			if (!leftMouseDown)
			{
				mouseClickX = mouseX;
	   			mouseClickY = mouseY;
			}
   		
   		    leftMouseDown = 1;
   		}
		
		if (state == GLUT_UP)
		{
			leftMouseDown = 0;
		}
   }
}

int main(int argc, char** argv)
{
	// Create scene objects
    maze = new Maze();
	wall = new Wall();
	camera = new Camera();

    loadMaze();

	// Glut init
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize (500, 500); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("Walker");

	init ();

	// Callbacks
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(mouseMove);
	glutMainLoop();
   
	return 0;
}




